/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Tara Madhyastha (tara@cs.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 * SimpleSonicWidget.h: The most basic type of sonic widget.
 *
 */

#ifndef SimpleSonicWidget_h
#define SimpleSonicWidget_h

#include "BaseSonicWidget.h"
#include "SoundControlFile.h"

class SimpleSonicWidget : public BaseSonicWidget {
  /*
   * SimpleSonicWidget is the most primitive type of Sonic Widget. A SimpleSonicWidget
   * encapsulates a single Sound Control File, which specifies:
   *       (1) What host and device(port name) the file was created for.
   *       (2) The messages to be sent, including variable, fixed, and 
   *           configurable parameters.
   * The "execute" procedure performs all possible mappings to variable
   * parameters and sends the messages specified in the file to the sound
   * device, so that they are played. 
   *
   * This is the simplest kind of behavior. 
   */

private:
       
       
public:
  /* Method SimpleSonicWidget: The SimpleSonicWidget constructor */
  SimpleSonicWidget(char *name=""): BaseSonicWidget(name) { } ;

  /* Method execute: Execute the sonic widget with scalar or vector */
  virtual void  execute(double d);
  virtual void  execute(double *a, int length);

  /* Method initialize: Initialize the sonic widget from a widget control   *
   *                    file that has been positioned past the sonic widget *
   *                    class identifier.                                   */
  virtual Bool_ initialize(istream &is);


  /* Method myClassName: Return the name of the widget class */
  virtual const char* myClassName() const { return("SimpleSonicWidget"); } ;

  /* Method printOn: Stream output function */
  virtual void printOn(ostream &os = cout);

};

inline ostream &operator<<(ostream &os, SimpleSonicWidget &obj)
{
	obj.printOn(os);
	return os;
}

#endif
