/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Ruth Aydt (aydt@cs.uiuc.edu)
 * Contributing Author: Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 */
/*
 * GeneralDialogBox.h: Implements a Motif-style GeneralDialog box with
 *		       Full Application Modality.
 *
 * $Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/System/Includes/RCS/GeneralDialogBox.h,v 1.13 1994/02/25 04:31:21 aydt Exp $	
 */

#ifndef GeneralDialogBox_h
#define GeneralDialogBox_h

#include <Xm/Xm.h>
#include "Obj.h"

class GeneralDialogBox : public Obj {
/*
 *	The GeneralDialogBox is used to provide user feedback via a popup
 *	dialog box.  It is used throughout the Pablo interface.
 */

private:
	enum DialogType {
		ErrorDialog,
		InformationDialog,
		MessageDialog,
		QuestionDialog,
		WarningDialog };

	struct cbInfo {
		GeneralDialogBox *dp;
		int	         button;
	} ;

	static const char *const MY_CLASS;	// Our class name
							
	/* Method static_cb:			This is a static method that
	*					handles all the callbacks for
	*					all instances of the class    */
	static void static_cb( Widget widget, cbInfo *cbi, 
			       XmAnyCallbackStruct *cbs );
	
	cbInfo	 cbinfo;			// Struct for callback info
	Widget	 dialog;			// The Widget

	/* Method _run:				The method that actually runs
	*					the dialog box.  It returns
	*					1 for OK, 2 for Cancel and 3
	*					for everything else.          */
	int _run( DialogType myType, const char *msgBuf );

public:
	/* Method GeneralDialogBox:		The constructor.	      */
	GeneralDialogBox();

	/* Method ~GeneralDialogBox:		The destructor.		      */
	~GeneralDialogBox();

	/* Method callback:			The per-instance method for
	*					handling callbacks.  This is
	*					called via static_cb.	      */
	void callback( Widget widget, cbInfo *cbi, XmAnyCallbackStruct *cbs );

	/* Method run:				Formats a message and displays 
	*					it in a message dialog.       */
	void run( char *fmt, ... );

	/* Method runError:			Formats a message and displays 
	*					it in an error dialog.        */
	void runError( char *fmt, ... );

	/* Method runInfo:			Formats a message and displays 	
	*					it in an information dialog.  */
	void runInfo( char *fmt, ... );

	/* Method runQuestion:			Formats a message and displays 
	*					it in a question dialog. 
	*					TRUE_ is returned if OK was
	*					selected; else FALSE_.        */
	Boolean_ runQuestion( char *fmt, ... );

	/* Method runWarning:			Formats a message and displays 
	*					it in a warning dialog.    */
	void runWarning( char *fmt, ... );
};

#endif GeneralDialogBox_h
