/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Philip C. Roth (proth@cs.uiuc.edu)
 *
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 * KiviatUtilDisplay.h: Shows utilizations on kiviat diagrams
 *
 * $Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/System/Includes/RCS/KiviatUtilDisplay.h,v 1.9 1994/02/25 04:31:42 aydt Exp $
 */

#ifndef KiviatUtilDisplay_h
#define	KiviatUtilDisplay_h

#include <X11/Intrinsic.h>
#include "UtilDisplay.h"

extern "C" {
#include "Kiviat.h"
}

class KiviatUtilDisplay : public UtilDisplay {
/*
 * KiviatUtilDisplay displays utilizations on
 * a single kiviat diagram
 */

private:

	int	numAxes;		// number of axes in the wheel
	int	*vals;			// the utilization values shown


	/* Method _handleResize:	Manages UtilDisplay
	 *				dialog resizing			*/
	virtual void _handleResize( Widget w );


public:

	/* Method KiviatUtilDisplay:	The class constructor.
	 *				A new KiviatUtilDisplay is
	 *				set using the given name, with
	 *				FU parent ufu, and X widget
	 *				parent par			*/
	KiviatUtilDisplay( char*, UtilFU*, Widget );


	/* Method ~KiviatUtilDisplay:	The class destructor		*/
	virtual ~KiviatUtilDisplay( void );


	/* Method changeValue:		Updates the utilization value 
	 *				for PE number proc to util	*/
	virtual void	changeValue( int proc, float util );


	/* Method createPerfSetup:	Creates a single setup for utilization
	 *				shown on a kiviat performance
	 *				widget				*/
	virtual	Widget	createPerfSetup( const char *label );


	/* Method getNumAxes:		Returns the number of axes on 
	 *				the kiviat graph		*/
	int	getNumAxes( void );


	/* Method getType:		Returns a character string describing
	 *				the type of this UtilDisplay	*/
	virtual const char * const getType( void ) const;


	/* Method updatePerfWidgets:	Modifies the existing UtilDisplay
	 *				dialog to show the updated list
	 *				of individual selections or
	 *				aggregates			*/
	virtual void	updatePerfWidgets( void );

};

inline
int
KiviatUtilDisplay::getNumAxes( void )
{
	return numAxes;
}

#endif KiviatUtilDisplay_h
