/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Allen D. Malony (malony@uicsrd.csrd.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 *	$Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/Widgets/Includes/RCS/BargraphP.h,v 1.8 1994/02/25 04:47:34 aydt Exp $
 */

#ifndef _BargraphP_h
#define _BargraphP_h

#include <X11/CoreP.h>
#include "Bargraph.h"

#define DEFAULT_LENGTH		200
#define DEFAULT_THICKNESS	50

#define EVENT_VALUE_UPDATE	1
#define EVENT_COLOR_UPDATE	2
#define EVENT_PIXMAP_UPDATE	3
#define EVENT_REDISPLAY		4


typedef struct {
  /* public */
  int		sticky_hold;		/* sticky hold interval		*/
  int		value;			/* current value		*/
  int		min_value, max_value;	/* minimum and maximum value	*/
  Pixel		fill_color;		/* fill color			*/
  Pixel		sticky_color;		/* sticky color			*/
  Pixmap	fill_pixmap;		/* fill pixmap			*/
  XtOrientation	orientation;		/* horizontal or vertical	*/
  XtCallbackList select;		/* callback for select		*/
  /* private */
  int		value_select;		/* selected value		*/
  int		length;			/* either height or width	*/
  int		thickness;		/* either width or height	*/
  int		sticky_held;		/* sticky held interval		*/
  int		old_value;		/* old percent filled		*/
  int		sticky_value;		/* sticky value			*/
  GC		eraseGC;		/* gc for erasing		*/
  GC		fill_colorGC;		/* gc for color filling		*/
  GC		fill_pixmapGC;		/* gc for pixmap filling	*/
  GC		stickyGC;		/* gc for sticky color		*/
} BargraphPart;

typedef struct _BargraphRec {
  CorePart core;
  BargraphPart bargraph;
} BargraphRec;

typedef struct _BargraphClass {
  int make_compiler_happy;
} BargraphClassPart;

typedef struct _BargraphClassRec {
  CoreClassPart core_class;
  BargraphClassPart bargraph_class;
} BargraphClassRec;

extern BargraphClassRec bargraphClassRec;

#endif _Bargraphp_h
